@@ -2,22 +2,29 @@
 Main.py
 """
 from __future__ import annotations
+
 import argparse
 import asyncio
 import json
 import os
 import random
+import ssl
 import uuid
-import urllib.parse
 from enum import Enum
 from typing import Generator
 from typing import Literal
 from typing import Optional
 from typing import Union
 
+import certifi
 import httpx
-import aiohttp
-from aiohttp import ClientWebSocketResponse
+import websockets.client as websockets
+from prompt_toolkit import PromptSession
+from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
+from prompt_toolkit.completion import WordCompleter
+from prompt_toolkit.history import InMemoryHistory
+from rich.live import Live
+from rich.markdown import Markdown
 
 DELIMITER = "\x1e"
 
@@ -73,14 +80,17 @@ HEADERS_INIT_CONVER = {
     "x-edge-shopping-flag": "1",
 }
 
+ssl_context = ssl.create_default_context()
+ssl_context.load_verify_locations(certifi.where())
+
 
 class NotAllowedToAccess(Exception):
     pass
 
 
 class ConversationStyle(Enum):
-    creative = "h3imaginative"
-    balanced = "harmonyv3"
+    creative = "h3relaxedimg"
+    balanced = "galileo"
     precise = "h3precise"
 
 
@@ -120,7 +130,7 @@ class ChatHubRequest:
         self,
         prompt: str,
         conversation_style: CONVERSATION_STYLE_TYPE,
-        options: Optional[list] = None,
+        options: list | None = None,
     ) -> None:
         """
         Updates request object
@@ -173,23 +183,14 @@ class Conversation:
     Conversation API
     """
 
-    def __init__(
-        self,
-        cookiePath: str = "",
-        cookies: Optional[dict] = None,
-        proxy: Optional[str] = None,
-    ) -> None:
+    def __init__(self, cookiePath: str = "", cookies: dict | None = None) -> None:
         self.struct: dict = {
             "conversationId": None,
             "clientId": None,
             "conversationSignature": None,
             "result": {"value": "Success", "message": None},
         }
-        self.proxy = proxy
-        proxies = {
-            "all://": proxy,
-        }
-        self.session = httpx.Client(proxies=proxies)
+        self.session = httpx.Client()
         self.session.headers.update(
             {
                 "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
@@ -233,8 +234,7 @@ class ChatHub:
     """
 
     def __init__(self, conversation: Conversation) -> None:
-        self.session: Optional[aiohttp.ClientSession] = None
-        self.wss: Optional[ClientWebSocketResponse] = None
+        self.wss: websockets.WebSocketClientProtocol | None = None
         self.request: ChatHubRequest
         self.loop: bool
         self.task: asyncio.Task
@@ -243,7 +243,6 @@ class ChatHub:
             client_id=conversation.struct["clientId"],
             conversation_id=conversation.struct["conversationId"],
         )
-        self.proxy = conversation.proxy
 
     async def ask_stream(
         self,
@@ -253,42 +252,41 @@ class ChatHub:
         """
         Ask a question to the bot
         """
+        if self.wss:
+            if not self.wss.closed:
+                await self.wss.close()
         # Check if websocket is closed
-        if not self.session:
-            self.session = aiohttp.ClientSession()
-        if not self.wss:
-            self.wss = await self.session.ws_connect(
-                url="wss://sydney.bing.com/sydney/ChatHub",
-                headers=HEADERS,
-                proxy=self.proxy
-            )
-            await self.__initial_handshake()
-
+        self.wss = await websockets.connect(
+            "wss://sydney.bing.com/sydney/ChatHub",
+            extra_headers=HEADERS,
+            max_size=None,
+            ssl=ssl_context,
+        )
+        await self.__initial_handshake()
         # Construct a ChatHub request
         self.request.update(prompt=prompt, conversation_style=conversation_style)
         # Send request
-        await self.wss.send_str(append_identifier(self.request.struct))
+        await self.wss.send(append_identifier(self.request.struct))
         final = False
         while not final:
-            msg = await self.wss.receive()
-            objects = msg.data.split(DELIMITER)
+            objects = str(await self.wss.recv()).split(DELIMITER)
             for obj in objects:
                 if obj is None or obj == "":
                     continue
                 response = json.loads(obj)
                 if response.get("type") == 1 and response["arguments"][0].get(
-                    "messages"
+                    "messages",
                 ):
                     yield False, response["arguments"][0]["messages"][0][
                         "adaptiveCards"
-                    ][0]["body"][0]["text"]
+                    ][0]["body"][0].get("text")
                 elif response.get("type") == 2:
                     final = True
                     yield True, response
 
     async def __initial_handshake(self):
-        await self.wss.send_str(append_identifier({"protocol": "json", "version": 1}))
-        await self.wss.receive()
+        await self.wss.send(append_identifier({"protocol": "json", "version": 1}))
+        await self.wss.recv()
 
     async def close(self):
         """
@@ -296,8 +294,6 @@ class ChatHub:
         """
         if self.wss and not self.wss.closed:
             await self.wss.close()
-        if self.session and not self.session:
-            await self.session.close()
 
 
 class Chatbot:
@@ -305,18 +301,10 @@ class Chatbot:
     Combines everything to make it seamless
     """
 
-    def __init__(
-        self,
-        cookiePath: str = "",
-        cookies: Optional[dict] = None,
-        proxy: Optional[str] = None,
-    ) -> None:
+    def __init__(self, cookiePath: str = "", cookies: dict | None = None) -> None:
         self.cookiePath: str = cookiePath
-        self.cookies: Optional[dict] = cookies
-        self.chat_hub: ChatHub = ChatHub(
-            Conversation(self.cookiePath, self.cookies, proxy)
-        )
-        self.proxy = proxy
+        self.cookies: dict | None = cookies
+        self.chat_hub: ChatHub = ChatHub(Conversation(self.cookiePath, self.cookies))
 
     async def ask(
         self,
@@ -332,7 +320,7 @@ class Chatbot:
         ):
             if final:
                 return response
-        await self.chat_hub.wss.close()
+        self.chat_hub.wss.close()
 
     async def ask_stream(
         self,
@@ -359,50 +347,42 @@ class Chatbot:
         Reset the conversation
         """
         await self.close()
-        self.chat_hub = ChatHub(Conversation(self.cookiePath, self.cookies, self.proxy))
+        self.chat_hub = ChatHub(Conversation(self.cookiePath, self.cookies))
 
 
-def get_input(prompt):
+async def get_input_async(
+    session: PromptSession = None,
+    completer: WordCompleter = None,
+) -> str:
     """
-    Multi-line input function
+    Multiline input function.
     """
-    # Display the prompt
-    print(prompt, end="")
-
-    if args.enter_once:
-        user_input = input()
-        print()
-        return user_input
-
-    # Initialize an empty list to store the input lines
-    lines = []
-
-    # Read lines of input until the user enters an empty line
-    while True:
-        line = input()
-        if line == "":
-            break
-        lines.append(line)
-
-    # Join the lines, separated by newlines, and store the result
-    user_input = "\n".join(lines)
-
-    # Return the input
-    return user_input
+    return await session.prompt_async(
+        completer=completer,
+        multiline=True,
+        auto_suggest=AutoSuggestFromHistory(),
+    )
 
 
-async def main(args: argparse.Namespace):
+def create_session() -> PromptSession:
+    return PromptSession(history=InMemoryHistory())
+
+
+async def main():
     """
     Main function
     """
     print("Initializing...")
-    bot = Chatbot(proxy=args.proxy)
+    print("Enter `alt+enter` or `escape+enter` to send a message")
+    bot = Chatbot()
+    session = create_session()
     while True:
-        prompt = get_input("\nYou:\n")
-        if prompt == "!exit":
-            await bot.close()
+        print("\nYou:")
+        question = await get_input_async(session=session)
+        print()
+        if question == "!exit":
             break
-        elif prompt == "!help":
+        elif question == "!help":
             print(
                 """
             !help - Show this help message
@@ -411,17 +391,15 @@ async def main(args: argparse.Namespace):
             """,
             )
             continue
-        elif prompt == "!reset":
+        elif question == "!reset":
             await bot.reset()
             continue
         print("Bot:")
         if args.no_stream:
             print(
-                Markdown(
-                    (await bot.ask(prompt=prompt, conversation_style=args.style))[
-                        "item"
-                    ]["messages"][1]["adaptiveCards"][0]["body"][0]["text"],
-                )
+                (await bot.ask(prompt=question, conversation_style=args.style))["item"][
+                    "messages"
+                ][1]["adaptiveCards"][0]["body"][0]["text"],
             )
         else:
             if args.rich:
@@ -429,7 +407,7 @@ async def main(args: argparse.Namespace):
                 md = Markdown("")
                 with Live(md, auto_refresh=False) as live:
                     async for final, response in bot.ask_stream(
-                        prompt=prompt,
+                        prompt=question,
                         conversation_style=args.style,
                     ):
                         if not final:
@@ -442,13 +420,13 @@ async def main(args: argparse.Namespace):
             else:
                 wrote = 0
                 async for final, response in bot.ask_stream(
-                    prompt=prompt,
+                    prompt=question,
                     conversation_style=args.style,
                 ):
                     if not final:
                         print(response[wrote:], end="", flush=True)
                         wrote = len(response)
-            print()
+                print()
     await bot.close()
 
 
@@ -462,7 +440,6 @@ if __name__ == "__main__":
         !help for help
 
         Type !exit to exit
-        Enter twice to send message or set --enter-once to send one line message
     """,
     )
     parser = argparse.ArgumentParser()
@@ -474,7 +451,6 @@ if __name__ == "__main__":
         choices=["creative", "balanced", "precise"],
         default="balanced",
     )
-    parser.add_argument("--proxy", type=str, required=False)
     parser.add_argument(
         "--cookie-file",
         type=str,
@@ -483,14 +459,5 @@ if __name__ == "__main__":
     )
     args = parser.parse_args()
     os.environ["COOKIE_FILE"] = args.cookie_file
-    if args.rich:
-        try:
-            from rich import print
-            from rich.live import Live
-            from rich.markdown import Markdown
-        except ModuleNotFoundError:
-            print("You'll need to install `rich` for this feature: `pip install rich`")
-            exit(1)
-    else:
-        import sys
-    asyncio.run(main(args))
+    args = parser.parse_args()
+    asyncio.run(main())
